﻿namespace HelloOwin
{
    using System;
    using System.Globalization;
    using System.Linq;
    using System.Threading.Tasks;
    using Microsoft.Owin;
    using Microsoft.Owin.Hosting;
    using Owin;
    using OwinFunc = System.Func<System.Collections.Generic.IDictionary<string, object>, System.Threading.Tasks.Task>;

    static class Program
    {
        class Elwin : OwinMiddleware
        {
            public Elwin(OwinMiddleware next) : base(next)
            {
            }

            public override Task Invoke(OwinRequest request, OwinResponse response)
            {
                return InvokeImpl(request, response);
            }

            async Task InvokeImpl(OwinRequest request, OwinResponse response)
            {
                //await Task.Yield();
                await Next.Invoke(request, response);
            }
        }
        static void Main(string[] args)
        {
            var portArg = args.FirstOrDefault(arg => !string.IsNullOrWhiteSpace(arg));
            using (WebApp.Start(portArg != null 
                                ? int.Parse(portArg, NumberStyles.None, CultureInfo.InvariantCulture) 
                                : 50000, builder =>
            {
                builder.Use(typeof(Elwin));

                builder.Use((Func<OwinFunc, OwinFunc>) (next => env =>
                {
                    var tcs = new TaskCompletionSource<object>();
                    next(env).ContinueWith(t =>
                    {
                        if (t.IsCanceled) tcs.SetCanceled();
                        else if (t.IsFaulted)
                        {
                            var originalColor = Console.ForegroundColor;
                            Console.ForegroundColor = ConsoleColor.Red;
                            Console.Error.WriteLine(t.Exception);
                            Console.ForegroundColor = originalColor;
                            tcs.SetException(t.Exception);
                        }
                        else tcs.SetResult(null);
                    }); 
                    return tcs.Task;
                }));

                builder.Run((OwinFunc) (env =>
                {
                    var req = OwinRequest.Create();
                    Array.ForEach(env.Select(e => new { e.Key, e.Value }).ToArray(), Console.Error.WriteLine);
                    var tcs = new TaskCompletionSource<object>();
                    if (DateTime.Now.Second % 2 == 0)
                        tcs.SetResult(null);
                    else
                        tcs.SetException(new ApplicationException());
                    return tcs.Task;
                }));
            }))
            {
                Console.WriteLine("Press Enter to quit.");
                Console.ReadKey();
            }
        }
    }
}
